@@ -1,6 +1,6 @@
 import pytest
 
-from yarl import URL
+from yarl._path import normalize_path
 
 PATHS = [
     # No dots
@@ -32,5 +32,5 @@ PATHS = [
 
 
 @pytest.mark.parametrize("original,expected", PATHS)
-def test__normalize_path(original, expected):
-    assert URL._normalize_path(original) == expected
+def test_normalize_path(original: str, expected: str) -> None:
+    assert normalize_path(original) == expected
