@@ -5,48 +5,48 @@ from yarl import URL
 # build classmethod
 
 
-def test_build_without_arguments():
+def test_build_without_arguments() -> None:
     u = URL.build()
     assert str(u) == ""
 
 
-def test_build_simple():
+def test_build_simple() -> None:
     u = URL.build(scheme="http", host="127.0.0.1")
     assert str(u) == "http://127.0.0.1"
 
 
-def test_url_build_ipv6():
+def test_url_build_ipv6() -> None:
     u = URL.build(scheme="http", host="::1")
     assert str(u) == "http://[::1]"
 
 
-def test_url_build_ipv6_brackets_encoded():
+def test_url_build_ipv6_brackets_encoded() -> None:
     u = URL.build(scheme="http", host="[::1]", encoded=True)
     assert str(u) == "http://[::1]"
 
 
-def test_url_build_ipv6_brackets_not_encoded():
+def test_url_build_ipv6_brackets_not_encoded() -> None:
     u = URL.build(scheme="http", host="::1", encoded=False)
     assert str(u) == "http://[::1]"
 
 
-def test_url_ipv4_in_ipv6():
+def test_url_ipv4_in_ipv6() -> None:
     u = URL.build(scheme="http", host="2001:db8:122:344::192.0.2.33")
     assert str(u) == "http://[2001:db8:122:344::c000:221]"
 
 
-def test_build_with_scheme():
+def test_build_with_scheme() -> None:
     u = URL.build(scheme="blob", path="path")
     assert str(u) == "blob:path"
 
 
-def test_build_with_host():
+def test_build_with_host() -> None:
     u = URL.build(host="127.0.0.1")
     assert str(u) == "//127.0.0.1"
     assert u == URL("//127.0.0.1")
 
 
-def test_build_with_scheme_and_host():
+def test_build_with_scheme_and_host() -> None:
     u = URL.build(scheme="http", host="127.0.0.1")
     assert str(u) == "http://127.0.0.1"
     assert u == URL("http://127.0.0.1")
@@ -70,23 +70,22 @@ def test_build_with_scheme_and_host():
         ),
     ],
 )
-def test_build_with_port(port, exc, match):
-    print(match)
+def test_build_with_port(port: int, exc: type[Exception], match: str) -> None:
     with pytest.raises(exc, match=match):
         URL.build(port=port)
 
 
-def test_build_with_user():
+def test_build_with_user() -> None:
     u = URL.build(scheme="http", host="127.0.0.1", user="foo")
     assert str(u) == "http://foo@127.0.0.1"
 
 
-def test_build_with_user_password():
+def test_build_with_user_password() -> None:
     u = URL.build(scheme="http", host="127.0.0.1", user="foo", password="bar")
     assert str(u) == "http://foo:bar@127.0.0.1"
 
 
-def test_build_with_query_and_query_string():
+def test_build_with_query_and_query_string() -> None:
     with pytest.raises(ValueError):
         URL.build(
             scheme="http",
@@ -101,7 +100,7 @@ def test_build_with_query_and_query_string():
         )
 
 
-def test_build_with_all():
+def test_build_with_all() -> None:
     u = URL.build(
         scheme="http",
         host="127.0.0.1",
@@ -115,7 +114,7 @@ def test_build_with_all():
     assert str(u) == "http://foo:bar@127.0.0.1:8000/index.html?arg=value1#top"
 
 
-def test_build_with_authority_and_host():
+def test_build_with_authority_and_host() -> None:
     with pytest.raises(ValueError):
         URL.build(authority="host.com", host="example.com")
 
@@ -131,7 +130,7 @@ def test_build_with_authority_and_host():
         *(("other_gen_delim_" + c, False) for c in "/?#[]"),
     ],
 )
-def test_build_with_invalid_host(host: str, is_authority: bool):
+def test_build_with_invalid_host(host: str, is_authority: bool) -> None:
     match = r"Host '[^']+' cannot contain '[^']+' \(at position \d+\)"
     if is_authority:
         match += ", if .* use 'authority' instead of 'host'"
@@ -139,63 +138,63 @@ def test_build_with_invalid_host(host: str, is_authority: bool):
         URL.build(host=host)
 
 
-def test_build_with_authority():
+def test_build_with_authority() -> None:
     url = URL.build(scheme="http", authority="степан:bar@host.com:8000", path="/path")
     assert (
         str(url) == "http://%D1%81%D1%82%D0%B5%D0%BF%D0%B0%D0%BD:bar@host.com:8000/path"
     )
 
 
-def test_build_with_authority_no_leading_flash():
+def test_build_with_authority_no_leading_flash() -> None:
     msg = r"Path in a URL with authority should start with a slash \('/'\) if set"
     with pytest.raises(ValueError, match=msg):
         URL.build(scheme="http", authority="степан:bar@host.com:8000", path="path")
 
 
-def test_build_with_authority_without_encoding():
+def test_build_with_authority_without_encoding() -> None:
     url = URL.build(
         scheme="http", authority="foo:bar@host.com:8000", path="path", encoded=True
     )
     assert str(url) == "http://foo:bar@host.com:8000/path"
 
 
-def test_build_with_authority_empty_host_no_scheme():
+def test_build_with_authority_empty_host_no_scheme() -> None:
     url = URL.build(authority="", path="path")
     assert str(url) == "path"
 
 
-def test_build_with_authority_and_only_user():
+def test_build_with_authority_and_only_user() -> None:
     url = URL.build(scheme="https", authority="user:@foo.com", path="/path")
     assert str(url) == "https://user:@foo.com/path"
 
 
-def test_build_with_authority_with_port():
+def test_build_with_authority_with_port() -> None:
     url = URL.build(scheme="https", authority="foo.com:8080", path="/path")
     assert str(url) == "https://foo.com:8080/path"
 
 
-def test_build_with_authority_with_ipv6():
+def test_build_with_authority_with_ipv6() -> None:
     url = URL.build(scheme="https", authority="[::1]", path="/path")
     assert str(url) == "https://[::1]/path"
 
 
-def test_build_with_authority_with_ipv6_and_port():
+def test_build_with_authority_with_ipv6_and_port() -> None:
     url = URL.build(scheme="https", authority="[::1]:81", path="/path")
     assert str(url) == "https://[::1]:81/path"
 
 
-def test_query_str():
+def test_query_str() -> None:
     u = URL.build(scheme="http", host="127.0.0.1", path="/", query_string="arg=value1")
     assert str(u) == "http://127.0.0.1/?arg=value1"
 
 
-def test_query_dict():
+def test_query_dict() -> None:
     u = URL.build(scheme="http", host="127.0.0.1", path="/", query=dict(arg="value1"))
 
     assert str(u) == "http://127.0.0.1/?arg=value1"
 
 
-def test_build_path_quoting():
+def test_build_path_quoting() -> None:
     u = URL.build(
         scheme="http",
         host="127.0.0.1",
@@ -211,7 +210,7 @@ def test_build_path_quoting():
     )
 
 
-def test_build_query_quoting():
+def test_build_query_quoting() -> None:
     u = URL.build(
         scheme="http",
         host="127.0.0.1",
@@ -227,18 +226,18 @@ def test_build_query_quoting():
     )
 
 
-def test_build_query_only():
+def test_build_query_only() -> None:
     u = URL.build(query={"key": "value"})
 
     assert str(u) == "?key=value"
 
 
-def test_build_drop_dots():
+def test_build_drop_dots() -> None:
     u = URL.build(scheme="http", host="example.com", path="/path/../to")
     assert str(u) == "http://example.com/to"
 
 
-def test_build_encode():
+def test_build_encode() -> None:
     u = URL.build(
         scheme="http",
         host="оун-упа.укр",
@@ -255,7 +254,7 @@ def test_build_encode():
     assert str(u) == expected
 
 
-def test_build_already_encoded():
+def test_build_already_encoded() -> None:
     # resulting URL is invalid but not encoded
     u = URL.build(
         scheme="http",
@@ -268,7 +267,7 @@ def test_build_already_encoded():
     assert str(u) == "http://оун-упа.укр/шлях/криївка?ключ=знач#фраг"
 
 
-def test_build_already_encoded_username_password():
+def test_build_already_encoded_username_password() -> None:
     u = URL.build(
         scheme="http",
         host="x.org",
@@ -283,7 +282,7 @@ def test_build_already_encoded_username_password():
     assert u.host_subcomponent == "x.org"
 
 
-def test_build_already_encoded_empty_host():
+def test_build_already_encoded_empty_host() -> None:
     u = URL.build(
         host="",
         path="/x/y/z",
@@ -295,7 +294,7 @@ def test_build_already_encoded_empty_host():
     assert u.host_subcomponent is None
 
 
-def test_build_percent_encoded():
+def test_build_percent_encoded() -> None:
     u = URL.build(
         scheme="http",
         host="%2d.org",
@@ -323,7 +322,7 @@ def test_build_percent_encoded():
     assert u.fragment == "f%2d"
 
 
-def test_build_with_authority_percent_encoded():
+def test_build_with_authority_percent_encoded() -> None:
     u = URL.build(scheme="http", authority="u%2d:p%2d@%2d.org")
     assert str(u) == "http://u%252d:p%252d@%2d.org"
     assert u.raw_host == "%2d.org"
@@ -336,7 +335,7 @@ def test_build_with_authority_percent_encoded():
     assert u.authority == "u%2d:p%2d@%2d.org:80"
 
 
-def test_build_with_authority_percent_encoded_already_encoded():
+def test_build_with_authority_percent_encoded_already_encoded() -> None:
     u = URL.build(scheme="http", authority="u%2d:p%2d@%2d.org", encoded=True)
     assert str(u) == "http://u%2d:p%2d@%2d.org"
     assert u.raw_host == "%2d.org"
@@ -349,36 +348,44 @@ def test_build_with_authority_percent_encoded_already_encoded():
     assert u.raw_authority == "u%2d:p%2d@%2d.org"
 
 
-def test_build_with_authority_with_path_with_leading_slash():
+def test_build_with_authority_with_path_with_leading_slash() -> None:
     u = URL.build(scheme="http", host="example.com", path="/path_with_leading_slash")
     assert str(u) == "http://example.com/path_with_leading_slash"
 
 
-def test_build_with_authority_with_empty_path():
+def test_build_with_authority_with_empty_path() -> None:
     u = URL.build(scheme="http", host="example.com", path="")
     assert str(u) == "http://example.com"
 
 
-def test_build_with_authority_with_path_without_leading_slash():
+def test_build_with_authority_with_path_without_leading_slash() -> None:
     with pytest.raises(ValueError):
         URL.build(scheme="http", host="example.com", path="path_without_leading_slash")
 
 
-def test_build_with_none_host():
+def test_build_with_none_host() -> None:
     with pytest.raises(TypeError, match="NoneType is illegal for.*host"):
-        URL.build(scheme="http", host=None)
+        URL.build(scheme="http", host=None)  # type: ignore[arg-type]
 
 
-def test_build_with_none_path():
+def test_build_with_none_path() -> None:
     with pytest.raises(TypeError):
-        URL.build(scheme="http", host="example.com", path=None)
+        URL.build(scheme="http", host="example.com", path=None)  # type: ignore[arg-type]
 
 
-def test_build_with_none_query_string():
+def test_build_with_none_query_string() -> None:
     with pytest.raises(TypeError):
-        URL.build(scheme="http", host="example.com", query_string=None)
+        URL.build(scheme="http", host="example.com", query_string=None)  # type: ignore[arg-type]
 
 
-def test_build_with_none_fragment():
+def test_build_with_none_fragment() -> None:
     with pytest.raises(TypeError):
-        URL.build(scheme="http", host="example.com", fragment=None)
+        URL.build(scheme="http", host="example.com", fragment=None)  # type: ignore[arg-type]
+
+
+def test_build_uppercase_host() -> None:
+    u = URL.build(
+        host="UPPER.case",
+        encoded=False,
+    )
+    assert u.host == "upper.case"
