@@ -1,4 +1,4 @@
-import sys
+from urllib.parse import SplitResult
 
 import pytest
 
@@ -6,7 +6,7 @@ from yarl import URL
 
 
 class TestScheme:
-    def test_scheme_path(self):
+    def test_scheme_path(self) -> None:
         u = URL("scheme:path")
         assert u.scheme == "scheme"
         assert u.host is None
@@ -14,7 +14,7 @@ class TestScheme:
         assert u.query_string == ""
         assert u.fragment == ""
 
-    def test_scheme_path_other(self):
+    def test_scheme_path_other(self) -> None:
         u = URL("scheme:path:other")
         assert u.scheme == "scheme"
         assert u.host is None
@@ -22,7 +22,7 @@ class TestScheme:
         assert u.query_string == ""
         assert u.fragment == ""
 
-    def test_complex_scheme(self):
+    def test_complex_scheme(self) -> None:
         u = URL("allow+chars-33.:path")
         assert u.scheme == "allow+chars-33."
         assert u.host is None
@@ -30,7 +30,7 @@ class TestScheme:
         assert u.query_string == ""
         assert u.fragment == ""
 
-    def test_scheme_only(self):
+    def test_scheme_only(self) -> None:
         u = URL("simple:")
         assert u.scheme == "simple"
         assert u.host is None
@@ -38,25 +38,15 @@ class TestScheme:
         assert u.query_string == ""
         assert u.fragment == ""
 
-    def test_no_scheme1(self):
+    def test_no_scheme1(self) -> None:
         u = URL("google.com:80")
-        # See: https://bugs.python.org/issue27657
-        if (
-            sys.version_info[:3] == (3, 7, 6)
-            or sys.version_info[:3] == (3, 8, 1)
-            or sys.version_info >= (3, 9, 0)
-        ):
-            assert u.scheme == "google.com"
-            assert u.host is None
-            assert u.path == "80"
-        else:
-            assert u.scheme == ""
-            assert u.host is None
-            assert u.path == "google.com:80"
+        assert u.scheme == "google.com"
+        assert u.host is None
+        assert u.path == "80"
         assert u.query_string == ""
         assert u.fragment == ""
 
-    def test_no_scheme2(self):
+    def test_no_scheme2(self) -> None:
         u = URL("google.com:80/root")
         assert u.scheme == "google.com"
         assert u.host is None
@@ -64,7 +54,7 @@ class TestScheme:
         assert u.query_string == ""
         assert u.fragment == ""
 
-    def test_not_a_scheme1(self):
+    def test_not_a_scheme1(self) -> None:
         u = URL("not_cheme:path")
         assert u.scheme == ""
         assert u.host is None
@@ -72,7 +62,7 @@ class TestScheme:
         assert u.query_string == ""
         assert u.fragment == ""
 
-    def test_not_a_scheme2(self):
+    def test_not_a_scheme2(self) -> None:
         u = URL("signals37:book")
         assert u.scheme == "signals37"
         assert u.host is None
@@ -80,7 +70,7 @@ class TestScheme:
         assert u.query_string == ""
         assert u.fragment == ""
 
-    def test_scheme_rel_path1(self):
+    def test_scheme_rel_path1(self) -> None:
         u = URL(":relative-path")
         assert u.scheme == ""
         assert u.host is None
@@ -88,7 +78,7 @@ class TestScheme:
         assert u.query_string == ""
         assert u.fragment == ""
 
-    def test_scheme_rel_path2(self):
+    def test_scheme_rel_path2(self) -> None:
         u = URL(":relative/path")
         assert u.scheme == ""
         assert u.host is None
@@ -96,7 +86,7 @@ class TestScheme:
         assert u.query_string == ""
         assert u.fragment == ""
 
-    def test_scheme_weird(self):
+    def test_scheme_weird(self) -> None:
         u = URL("://and-this")
         assert u.scheme == ""
         assert u.host is None
@@ -106,7 +96,7 @@ class TestScheme:
 
 
 class TestHost:
-    def test_canonical(self):
+    def test_canonical(self) -> None:
         u = URL("scheme://host/path")
         assert u.scheme == "scheme"
         assert u.host == "host"
@@ -114,7 +104,7 @@ class TestHost:
         assert u.query_string == ""
         assert u.fragment == ""
 
-    def test_absolute_no_scheme(self):
+    def test_absolute_no_scheme(self) -> None:
         u = URL("//host/path")
         assert u.scheme == ""
         assert u.host == "host"
@@ -122,7 +112,7 @@ class TestHost:
         assert u.query_string == ""
         assert u.fragment == ""
 
-    def test_absolute_no_scheme_complex_host(self):
+    def test_absolute_no_scheme_complex_host(self) -> None:
         u = URL("//host+path")
         assert u.scheme == ""
         assert u.host == "host+path"
@@ -130,7 +120,7 @@ class TestHost:
         assert u.query_string == ""
         assert u.fragment == ""
 
-    def test_absolute_no_scheme_simple_host(self):
+    def test_absolute_no_scheme_simple_host(self) -> None:
         u = URL("//host")
         assert u.scheme == ""
         assert u.host == "host"
@@ -138,7 +128,7 @@ class TestHost:
         assert u.query_string == ""
         assert u.fragment == ""
 
-    def test_weird_host(self):
+    def test_weird_host(self) -> None:
         u = URL("//this+is$also&host!")
         assert u.scheme == ""
         assert u.host == "this+is$also&host!"
@@ -146,7 +136,7 @@ class TestHost:
         assert u.query_string == ""
         assert u.fragment == ""
 
-    def test_scheme_no_host(self):
+    def test_scheme_no_host(self) -> None:
         u = URL("scheme:/host/path")
         assert u.scheme == "scheme"
         assert u.host is None
@@ -154,7 +144,7 @@ class TestHost:
         assert u.query_string == ""
         assert u.fragment == ""
 
-    def test_scheme_no_host2(self):
+    def test_scheme_no_host2(self) -> None:
         u = URL("scheme:///host/path")
         assert u.scheme == "scheme"
         assert u.host is None
@@ -162,7 +152,7 @@ class TestHost:
         assert u.query_string == ""
         assert u.fragment == ""
 
-    def test_no_scheme_no_host(self):
+    def test_no_scheme_no_host(self) -> None:
         u = URL("scheme//host/path")
         assert u.scheme == ""
         assert u.host is None
@@ -170,7 +160,7 @@ class TestHost:
         assert u.query_string == ""
         assert u.fragment == ""
 
-    def test_ipv4(self):
+    def test_ipv4(self) -> None:
         u = URL("//127.0.0.1/")
         assert u.scheme == ""
         assert u.host == "127.0.0.1"
@@ -178,7 +168,7 @@ class TestHost:
         assert u.query_string == ""
         assert u.fragment == ""
 
-    def test_ipv6(self):
+    def test_ipv6(self) -> None:
         u = URL("//[::1]/")
         assert u.scheme == ""
         assert u.host == "::1"
@@ -186,7 +176,7 @@ class TestHost:
         assert u.query_string == ""
         assert u.fragment == ""
 
-    def test_ipvfuture_address(self):
+    def test_ipvfuture_address(self) -> None:
         u = URL("//[v1.-1]/")
         assert u.scheme == ""
         assert u.host == "v1.-1"
@@ -196,7 +186,7 @@ class TestHost:
 
 
 class TestPort:
-    def test_canonical(self):
+    def test_canonical(self) -> None:
         u = URL("//host:80/path")
         assert u.scheme == ""
         assert u.host == "host"
@@ -205,7 +195,7 @@ class TestPort:
         assert u.query_string == ""
         assert u.fragment == ""
 
-    def test_no_path(self):
+    def test_no_path(self) -> None:
         u = URL("//host:80")
         assert u.scheme == ""
         assert u.host == "host"
@@ -214,35 +204,34 @@ class TestPort:
         assert u.query_string == ""
         assert u.fragment == ""
 
-    @pytest.mark.xfail(reason="https://github.com/aio-libs/yarl/issues/821")
-    def test_no_host(self):
-        u = URL("//:80")
+    def test_no_host(self) -> None:
+        u = URL("//:77")
         assert u.scheme == ""
         assert u.host == ""
-        assert u.port == 80
+        assert u.port == 77
         assert u.path == "/"
         assert u.query_string == ""
         assert u.fragment == ""
 
-    def test_double_port(self):
+    def test_double_port(self) -> None:
         with pytest.raises(ValueError):
             URL("//h:22:80/")
 
-    def test_bad_port(self):
+    def test_bad_port(self) -> None:
         with pytest.raises(ValueError):
             URL("//h:no/path")
 
-    def test_another_bad_port(self):
+    def test_another_bad_port(self) -> None:
         with pytest.raises(ValueError):
             URL("//h:22:no/path")
 
-    def test_bad_port_again(self):
+    def test_bad_port_again(self) -> None:
         with pytest.raises(ValueError):
             URL("//h:-80/path")
 
 
 class TestUserInfo:
-    def test_canonical(self):
+    def test_canonical(self) -> None:
         u = URL("sch://user@host/")
         assert u.scheme == "sch"
         assert u.user == "user"
@@ -251,7 +240,7 @@ class TestUserInfo:
         assert u.query_string == ""
         assert u.fragment == ""
 
-    def test_user_pass(self):
+    def test_user_pass(self) -> None:
         u = URL("//user:pass@host")
         assert u.scheme == ""
         assert u.user == "user"
@@ -261,7 +250,7 @@ class TestUserInfo:
         assert u.query_string == ""
         assert u.fragment == ""
 
-    def test_complex_userinfo(self):
+    def test_complex_userinfo(self) -> None:
         u = URL("//user:pas:and:more@host")
         assert u.scheme == ""
         assert u.user == "user"
@@ -271,7 +260,7 @@ class TestUserInfo:
         assert u.query_string == ""
         assert u.fragment == ""
 
-    def test_no_user(self):
+    def test_no_user(self) -> None:
         u = URL("//:pas:@host")
         assert u.scheme == ""
         assert u.user is None
@@ -281,7 +270,7 @@ class TestUserInfo:
         assert u.query_string == ""
         assert u.fragment == ""
 
-    def test_weird_user(self):
+    def test_weird_user(self) -> None:
         u = URL("//!($&')*+,;=@host")
         assert u.scheme == ""
         assert u.user == "!($&')*+,;="
@@ -291,7 +280,7 @@ class TestUserInfo:
         assert u.query_string == ""
         assert u.fragment == ""
 
-    def test_weird_user2(self):
+    def test_weird_user2(self) -> None:
         u = URL("//user@info@ya.ru")
         assert u.scheme == ""
         assert u.user == "user@info"
@@ -301,7 +290,7 @@ class TestUserInfo:
         assert u.query_string == ""
         assert u.fragment == ""
 
-    def test_weird_user3(self):
+    def test_weird_user3(self) -> None:
         u = URL("//%5Bsome%5D@host")
         assert u.scheme == ""
         assert u.user == "[some]"
@@ -313,7 +302,7 @@ class TestUserInfo:
 
 
 class TestQuery_String:
-    def test_simple(self):
+    def test_simple(self) -> None:
         u = URL("?query")
         assert u.scheme == ""
         assert u.user is None
@@ -323,7 +312,7 @@ class TestQuery_String:
         assert u.query_string == "query"
         assert u.fragment == ""
 
-    def test_scheme_query(self):
+    def test_scheme_query(self) -> None:
         u = URL("http:?query")
         assert u.scheme == "http"
         assert u.user is None
@@ -333,7 +322,7 @@ class TestQuery_String:
         assert u.query_string == "query"
         assert u.fragment == ""
 
-    def test_abs_url_query(self):
+    def test_abs_url_query(self) -> None:
         u = URL("//host?query")
         assert u.scheme == ""
         assert u.user is None
@@ -343,7 +332,7 @@ class TestQuery_String:
         assert u.query_string == "query"
         assert u.fragment == ""
 
-    def test_abs_url_path_query(self):
+    def test_abs_url_path_query(self) -> None:
         u = URL("//host/path?query")
         assert u.scheme == ""
         assert u.user is None
@@ -353,7 +342,7 @@ class TestQuery_String:
         assert u.query_string == "query"
         assert u.fragment == ""
 
-    def test_double_question_mark(self):
+    def test_double_question_mark(self) -> None:
         u = URL("//ho?st/path?query")
         assert u.scheme == ""
         assert u.user is None
@@ -363,7 +352,7 @@ class TestQuery_String:
         assert u.query_string == "st/path?query"
         assert u.fragment == ""
 
-    def test_complex_query(self):
+    def test_complex_query(self) -> None:
         u = URL("?a://b:c@d.e/f?g#h")
         assert u.scheme == ""
         assert u.user is None
@@ -373,7 +362,7 @@ class TestQuery_String:
         assert u.query_string == "a://b:c@d.e/f?g"
         assert u.fragment == "h"
 
-    def test_query_in_fragment(self):
+    def test_query_in_fragment(self) -> None:
         u = URL("#?query")
         assert u.scheme == ""
         assert u.user is None
@@ -385,7 +374,7 @@ class TestQuery_String:
 
 
 class TestFragment:
-    def test_simple(self):
+    def test_simple(self) -> None:
         u = URL("#frag")
         assert u.scheme == ""
         assert u.user is None
@@ -395,7 +384,7 @@ class TestFragment:
         assert u.query_string == ""
         assert u.fragment == "frag"
 
-    def test_scheme_frag(self):
+    def test_scheme_frag(self) -> None:
         u = URL("http:#frag")
         assert u.scheme == "http"
         assert u.user is None
@@ -405,7 +394,7 @@ class TestFragment:
         assert u.query_string == ""
         assert u.fragment == "frag"
 
-    def test_host_frag(self):
+    def test_host_frag(self) -> None:
         u = URL("//host#frag")
         assert u.scheme == ""
         assert u.user is None
@@ -415,7 +404,7 @@ class TestFragment:
         assert u.query_string == ""
         assert u.fragment == "frag"
 
-    def test_scheme_path_frag(self):
+    def test_scheme_path_frag(self) -> None:
         u = URL("//host/path#frag")
         assert u.scheme == ""
         assert u.user is None
@@ -425,7 +414,7 @@ class TestFragment:
         assert u.query_string == ""
         assert u.fragment == "frag"
 
-    def test_scheme_query_frag(self):
+    def test_scheme_query_frag(self) -> None:
         u = URL("//host?query#frag")
         assert u.scheme == ""
         assert u.user is None
@@ -435,7 +424,7 @@ class TestFragment:
         assert u.query_string == "query"
         assert u.fragment == "frag"
 
-    def test_host_frag_query(self):
+    def test_host_frag_query(self) -> None:
         u = URL("//ho#st/path?query")
         assert u.scheme == ""
         assert u.user is None
@@ -445,7 +434,7 @@ class TestFragment:
         assert u.query_string == ""
         assert u.fragment == "st/path?query"
 
-    def test_complex_frag(self):
+    def test_complex_frag(self) -> None:
         u = URL("#a://b:c@d.e/f?g#h")
         assert u.scheme == ""
         assert u.user is None
@@ -457,11 +446,21 @@ class TestFragment:
 
 
 class TestStripEmptyParts:
-    def test_all_empty(self):
+    def test_all_empty_http(self) -> None:
         with pytest.raises(ValueError):
-            URL("//@:?#")
+            URL("http://@:?#")
 
-    def test_path_only(self):
+    def test_all_empty(self) -> None:
+        u = URL("//@:?#")
+        assert u.scheme == ""
+        assert u.user is None
+        assert u.password is None
+        assert u.host == ""
+        assert u.path == ""
+        assert u.query_string == ""
+        assert u.fragment == ""
+
+    def test_path_only(self) -> None:
         u = URL("///path")
         assert u.scheme == ""
         assert u.user is None
@@ -471,7 +470,7 @@ class TestStripEmptyParts:
         assert u.query_string == ""
         assert u.fragment == ""
 
-    def test_empty_user(self):
+    def test_empty_user(self) -> None:
         u = URL("//@host")
         assert u.scheme == ""
         assert u.user is None
@@ -481,7 +480,7 @@ class TestStripEmptyParts:
         assert u.query_string == ""
         assert u.fragment == ""
 
-    def test_empty_port(self):
+    def test_empty_port(self) -> None:
         u = URL("//host:")
         assert u.scheme == ""
         assert u.user is None
@@ -491,7 +490,7 @@ class TestStripEmptyParts:
         assert u.query_string == ""
         assert u.fragment == ""
 
-    def test_empty_port_and_path(self):
+    def test_empty_port_and_path(self) -> None:
         u = URL("//host:/")
         assert u.scheme == ""
         assert u.user is None
@@ -501,7 +500,7 @@ class TestStripEmptyParts:
         assert u.query_string == ""
         assert u.fragment == ""
 
-    def test_empty_path_only(self):
+    def test_empty_path_only(self) -> None:
         u = URL("/")
         assert u.scheme == ""
         assert u.user is None
@@ -511,7 +510,7 @@ class TestStripEmptyParts:
         assert u.query_string == ""
         assert u.fragment == ""
 
-    def test_relative_path_only(self):
+    def test_relative_path_only(self) -> None:
         u = URL("path")
         assert u.scheme == ""
         assert u.user is None
@@ -521,7 +520,7 @@ class TestStripEmptyParts:
         assert u.query_string == ""
         assert u.fragment == ""
 
-    def test_path(self):
+    def test_path(self) -> None:
         u = URL("/path")
         assert u.scheme == ""
         assert u.user is None
@@ -531,7 +530,7 @@ class TestStripEmptyParts:
         assert u.query_string == ""
         assert u.fragment == ""
 
-    def test_empty_query_with_path(self):
+    def test_empty_query_with_path(self) -> None:
         u = URL("/path?")
         assert u.scheme == ""
         assert u.user is None
@@ -541,7 +540,7 @@ class TestStripEmptyParts:
         assert u.query_string == ""
         assert u.fragment == ""
 
-    def test_empty_query(self):
+    def test_empty_query(self) -> None:
         u = URL("?")
         assert u.scheme == ""
         assert u.user is None
@@ -551,7 +550,7 @@ class TestStripEmptyParts:
         assert u.query_string == ""
         assert u.fragment == ""
 
-    def test_empty_query_with_frag(self):
+    def test_empty_query_with_frag(self) -> None:
         u = URL("?#frag")
         assert u.scheme == ""
         assert u.user is None
@@ -561,7 +560,7 @@ class TestStripEmptyParts:
         assert u.query_string == ""
         assert u.fragment == "frag"
 
-    def test_path_empty_frag(self):
+    def test_path_empty_frag(self) -> None:
         u = URL("/path#")
         assert u.scheme == ""
         assert u.user is None
@@ -571,7 +570,7 @@ class TestStripEmptyParts:
         assert u.query_string == ""
         assert u.fragment == ""
 
-    def test_empty_path(self):
+    def test_empty_path(self) -> None:
         u = URL("#")
         assert u.scheme == ""
         assert u.user is None
@@ -580,3 +579,47 @@ class TestStripEmptyParts:
         assert u.path == ""
         assert u.query_string == ""
         assert u.fragment == ""
+
+
+@pytest.mark.parametrize(
+    ("scheme"),
+    [
+        ("http"),
+        ("https"),
+        ("ws"),
+        ("wss"),
+        ("ftp"),
+    ],
+)
+def test_schemes_that_require_host(scheme: str) -> None:
+    """Verify that schemes that require a host raise with empty host."""
+    expect = (
+        "Invalid URL: host is required for " f"absolute urls with the {scheme} scheme"
+    )
+    with pytest.raises(ValueError, match=expect):
+        URL(f"{scheme}://:1")
+
+
+@pytest.mark.parametrize(
+    ("url", "hostname", "hostname_without_brackets"),
+    [
+        ("http://[::1]", "[::1]", "::1"),
+        ("http://[::1]:8080", "[::1]", "::1"),
+        ("http://127.0.0.1:8080", "127.0.0.1", "127.0.0.1"),
+        (
+            "http://xn--jxagkqfkduily1i.eu",
+            "xn--jxagkqfkduily1i.eu",
+            "xn--jxagkqfkduily1i.eu",
+        ),
+    ],
+)
+def test_url_round_trips(
+    url: str, hostname: str, hostname_without_brackets: str
+) -> None:
+    """Verify that URLs round-trip correctly."""
+    parsed = URL(url)
+    assert SplitResult(*parsed._val).hostname == hostname_without_brackets
+    assert parsed.raw_host == hostname_without_brackets
+    assert parsed.host_subcomponent == hostname
+    assert str(parsed) == url
+    assert str(URL(str(parsed))) == url
