@@ -1,7 +1,7 @@
 import codecs
 import re
 from string import ascii_letters, ascii_lowercase, digits
-from typing import cast
+from typing import Union, overload
 
 BASCII_LOWERCASE = ascii_lowercase.encode("ascii")
 BPCT_ALLOWED = {f"%{i:02X}".encode("ascii") for i in range(256)}
@@ -33,7 +33,11 @@ class _Quoter:
         self._qs = qs
         self._requote = requote
 
-    def __call__(self, val: str) -> str:
+    @overload
+    def __call__(self, val: str) -> str: ...
+    @overload
+    def __call__(self, val: None) -> None: ...
+    def __call__(self, val: Union[str, None]) -> Union[str, None]:
         if val is None:
             return None
         if not isinstance(val, str):
@@ -115,21 +119,33 @@ class _Quoter:
 
 
 class _Unquoter:
-    def __init__(self, *, ignore: str = "", unsafe: str = "", qs: bool = False) -> None:
+    def __init__(
+        self,
+        *,
+        ignore: str = "",
+        unsafe: str = "",
+        qs: bool = False,
+        plus: bool = False,
+    ) -> None:
         self._ignore = ignore
         self._unsafe = unsafe
         self._qs = qs
+        self._plus = plus  # to match urllib.parse.unquote_plus
         self._quoter = _Quoter()
         self._qs_quoter = _Quoter(qs=True)
 
-    def __call__(self, val: str) -> str:
+    @overload
+    def __call__(self, val: str) -> str: ...
+    @overload
+    def __call__(self, val: None) -> None: ...
+    def __call__(self, val: Union[str, None]) -> Union[str, None]:
         if val is None:
             return None
         if not isinstance(val, str):
             raise TypeError("Argument should be str")
         if not val:
             return ""
-        decoder = cast(codecs.BufferedIncrementalDecoder, utf8_decoder())
+        decoder = utf8_decoder()
         ret = []
         idx = 0
         while idx < len(val):
@@ -173,7 +189,7 @@ class _Unquoter:
                 decoder.reset()
 
             if ch == "+":
-                if not self._qs or ch in self._unsafe:
+                if (not self._qs and not self._plus) or ch in self._unsafe:
                     ret.append("+")
                 else:
                     ret.append(" ")
